﻿/* Clockwise Matrix
 * Gordon Lee
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Test
{
    public class Matrix
    {
        // Exceptions
        public class BadMatrixException : Exception
        {
            public BadMatrixException(string str)
            {
                MessageBox.Show(str, "Bad Matrix");
            }
        }

        // Enums
        enum Direction { Right, Down, Left, Up };

        /* Set Matrix
         * @param List<List<double>> _matrix
         * Matrix to set
         */
        public void SetMatrix(List<List<double>> _matrix)
        {
            m_matrix.Clear();

            foreach (List<double> _row in _matrix)
            {
                List<double> _newRow = new List<double>();

                _newRow.AddRange(_row);

                m_matrix.Add(_newRow);
            }

            m_numRows = m_matrix.Count();
            m_numCols = m_matrix[0].Count();
        }

        /* Parse Matrix Input
         * @param string _matrixString
         * Matrix input string to parse
         */
        public void ParseMatrixInput(string _matrixString)
        {
            m_matrix.Clear();

            if (_matrixString.StartsWith("[") && _matrixString.EndsWith("]"))
            {
                int _maxCols = 0;
                // Remove [ and ]
                _matrixString = _matrixString.Substring(1, _matrixString.Length - 2);
                // Remove excess spaces and new lines
                _matrixString = _matrixString.Replace(" ", "");
                _matrixString = _matrixString.Replace("\r\n", "");

                // Split up rows by ';' character deliminator
                string[] _rowsStringArray = _matrixString.Split(';');
                foreach (string _rowString in _rowsStringArray)
                {
                    int _numCols = 0;
                    List<double> _newRow = new List<double>();

                    // Split up values by ',' character deliminator
                    string[] _valuesStringArray = _rowString.Split(',');

                    foreach (string _valueString in _valuesStringArray)
                    {
                        // Convert value string
                        double _value;
                        try
                        {
                            _value = Convert.ToDouble(_valueString);
                        }
                        catch (FormatException)
                        {
                            throw new BadMatrixException("Use of unacceptable characters");
                        }
                        _newRow.Add(_value);
                        ++_numCols;
                    }

                    // Check for number of columns
                    int _tempMaxCols = Math.Max(_numCols, _maxCols);
                    if (_maxCols == 0)
                    {
                        _maxCols = _tempMaxCols;
                    }
                    else
                    {
                        if (_maxCols != _numCols)
                        {
                            throw new BadMatrixException("Inconsistent number of columns");
                        }
                    }

                    m_matrix.Add(_newRow);
                }

                m_numRows = m_matrix.Count();
                m_numCols = m_matrix[0].Count();


            }
            else
            {
                throw new BadMatrixException("Does not start with [ and/or end with ]");
            }
        }

        /* Get Num Rows
         * @return int
         */
        public int GetNumRows()
        {
            return m_numRows;
        }

        /* Get Num Columns
         * @return int
         */
        public int GetNumColumns()
        {
            return m_numCols;
        }

        /* Get Value
         * @param int _r
         * Row
         * @param int _c
         * Column
         * @return double
         */
        public double GetValue(int _r, int _c)
        {
            return m_matrix[_r][_c];
        }

        /* Get Row
         * @param int _r
         * Row
         * @return List<double>
         */
        public List<double> GetRow(int _r)
        {
            return m_matrix[_r];
        }

        /* Get Column
         * @param int _c
         * Column
         * @return List<double>
         */
        public List<double> GetColumn(int _c)
        {
            List<double> _column = new List<double>();

            foreach (List<double> _row in m_matrix)
            {
                _column.Add(_row[_c]);
            }
            return _column;
        }

        /* Clear
         */
        public void Clear()
        {
            m_matrix.Clear();
            m_numRows = 0;
            m_numCols = 0;
        }

        /* Output Clockwise
         * Clockwise counts however many times the algorithm has gone right, down, left, and up.
         * With this information, the algorithm can head in a direction and know when to start and when to end
         * depending on these numbers.
         * For example heading right from the top left side, the algorithm must start from the very left but also
         * additionally right however many times the algorithm has already gone up so there are no repeating elements.
         * The direction right stops at the very right but stops earlier depending on however many times already gone down.
         * This repeats for all directions going clockwise until all elements from the matrix have been accounted for.
         * @return List<double>
         */
        public List<double> OutputClockwise()
        {
            List<double> _output = new List<double>();
            int _numEle = 0, _numRight = 0, _numDown = 0, _numLeft = 0, _numUp = 0;
            int _matrixSize = m_numRows * m_numCols;
            int _currDir = (int)Direction.Right;

            while (_numEle < _matrixSize)
            {
                switch (_currDir)
                {
                    // Heading Right from TopLeft Side
                    case (int)Direction.Right:
                        for (int i = 0 + _numUp; i < m_numCols - _numDown; ++i)
                        {
                            _output.Add(m_matrix[_numRight][i]);
                            ++_numEle;
                        }
                        _currDir = (int)Direction.Down;
                        ++_numRight;
                        break;
                    // Heading Down from TopRight Side
                    case (int)Direction.Down:
                        for (int i = 0 + _numRight; i < m_numRows - _numLeft; ++i)
                        {
                            _output.Add(m_matrix[i][m_numCols - 1 - _numDown]);
                            ++_numEle;
                        }
                        _currDir = (int)Direction.Left;
                        ++_numDown;
                        break;
                    // Heading Left from BottomRight Side
                    case (int)Direction.Left:
                        for (int i = m_numCols - 1 - _numDown; i >= 0 + _numUp; --i)
                        {
                            _output.Add(m_matrix[m_numRows - 1 - _numLeft][i]);
                            ++_numEle;
                        }
                        _currDir = (int)Direction.Up;
                        ++_numLeft;
                        break;
                    // Heading Up from BottomLeft Side
                    case (int)Direction.Up:
                        for (int i = m_numRows - 1 - _numLeft; i >= 0 + _numRight; --i)
                        {
                            _output.Add(m_matrix[i][_numUp]);
                            ++_numEle;
                        }
                        _currDir = (int)Direction.Right;
                        ++_numUp;
                        break;
                    default:
                        break;
                }
            }

            return _output;
        }

        // Member variables
        private List<List<double>> m_matrix = new List<List<double>>();
        private int m_numRows = 0;
        private int m_numCols = 0;
    }
}
